@@ -3,6 +3,7 @@
  *  oFono - Open Source Telephony
  *
  *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2018 Gemalto M2M
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,14 +24,17 @@
 #include <config.h>
 #endif
 
-#include <glib.h>
-#include <gatchat.h>
 #include <string.h>
 #include <stdlib.h>
+#include <errno.h>
+
+#include <glib.h>
+#include <gattty.h>
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 #include <ofono/log.h>
 #include <ofono/types.h>
+#include <ofono/modem.h>
 
 #include "atutil.h"
 #include "vendor.h"
@@ -115,13 +119,14 @@ gint at_util_call_compare(gconstpointer a, gconstpointer b)
 	return 0;
 }
 
-GSList *at_util_parse_clcc(GAtResult *result)
+GSList *at_util_parse_clcc(GAtResult *result, unsigned int *ret_mpty_ids)
 {
 	GAtResultIter iter;
 	GSList *l = NULL;
 	int id, dir, status, type;
 	ofono_bool_t mpty;
 	struct ofono_call *call;
+	unsigned int mpty_ids = 0;
 
 	g_at_result_iter_init(&iter, result);
 
@@ -173,8 +178,14 @@ GSList *at_util_parse_clcc(GAtResult *result)
 			call->clip_validity = 2;
 
 		l = g_slist_insert_sorted(l, call, at_util_call_compare);
+
+		if (mpty)
+			mpty_ids |= 1 << id;
 	}
 
+	if (ret_mpty_ids)
+		*ret_mpty_ids = mpty_ids;
+
 	return l;
 }
 
@@ -607,3 +618,174 @@ void at_util_sim_state_query_free(struct at_util_sim_state_query *req)
 
 	g_free(req);
 }
+
+/*
+ * CGCONTRDP returns addr + netmask in the same string in the form
+ * of "a.b.c.d.m.m.m.m" for IPv4.
+ * address/netmask must be able to hold
+ * 255.255.255.255 + null = 16 characters
+ */
+int at_util_get_ipv4_address_and_netmask(const char *addrnetmask,
+						char *address, char *netmask)
+{
+	const char *s = addrnetmask;
+	const char *net = NULL;
+
+	int ret = -EINVAL;
+	int i;
+
+	/* Count 7 dots for ipv4, less or more means error. */
+	for (i = 0; i < 9; i++, s++) {
+		s = strchr(s, '.');
+
+		if (!s)
+			break;
+
+		if (i == 3) {
+			/* set netmask ptr and break the string */
+			net = s + 1;
+		}
+	}
+
+	if (i == 7) {
+		memcpy(address, addrnetmask, net - addrnetmask);
+		address[net - addrnetmask - 1] = '\0';
+		strcpy(netmask, net);
+
+		ret = 0;
+	}
+
+	return ret;
+}
+
+/*
+ * CGCONTRDP returns addr + netmask in the same string in the form
+ * of "a1.a2.a3.a4.a5.a6.a7.a8.a9.a10.a11.a12.a13.a14.a15.a16.m1.m2.
+ * m3.m4.m5.m6.m7.m8.m9.m10.m11.m12.m13.m14.m15.m16" for IPv6.
+ * address/netmask must be able to hold 64 characters.
+ */
+int at_util_get_ipv6_address_and_netmask(const char *addrnetmask,
+						char *address, char *netmask)
+{
+	const char *s = addrnetmask;
+	const char *net = NULL;
+
+	int ret = -EINVAL;
+	int i;
+
+	/* Count 31 dots for ipv6, less or more means error. */
+	for (i = 0; i < 33; i++, s++) {
+		s = strchr(s, '.');
+
+		if (!s)
+			break;
+
+		if (i == 15) {
+			/* set netmask ptr and break the string */
+			net = s + 1;
+		}
+	}
+
+	if (i == 31) {
+		memcpy(address, addrnetmask, net - addrnetmask);
+		address[net - addrnetmask - 1] = '\0';
+		strcpy(netmask, net);
+
+		ret = 0;
+	}
+
+	return ret;
+}
+
+int at_util_gprs_auth_method_to_auth_prot(
+					enum ofono_gprs_auth_method auth_method)
+{
+	switch (auth_method) {
+	case OFONO_GPRS_AUTH_METHOD_PAP:
+		return 1;
+	case OFONO_GPRS_AUTH_METHOD_CHAP:
+		return 2;
+	case OFONO_GPRS_AUTH_METHOD_NONE:
+		return 0;
+	}
+
+	return 0;
+}
+
+const char *at_util_gprs_proto_to_pdp_type(enum ofono_gprs_proto proto)
+{
+	switch (proto) {
+	case OFONO_GPRS_PROTO_IPV6:
+		return "IPV6";
+	case OFONO_GPRS_PROTO_IPV4V6:
+		return "IPV4V6";
+		break;
+	case OFONO_GPRS_PROTO_IP:
+		return "IP";
+	}
+
+	return NULL;
+}
+
+char *at_util_get_cgdcont_command(guint cid, enum ofono_gprs_proto proto,
+								const char *apn)
+{
+	const char *pdp_type = at_util_gprs_proto_to_pdp_type(proto);
+
+	if (!apn)
+		return g_strdup_printf("AT+CGDCONT=%u", cid);
+
+	return g_strdup_printf("AT+CGDCONT=%u,\"%s\",\"%s\"", cid, pdp_type,
+									apn);
+}
+
+GAtChat *at_util_open_device(struct ofono_modem *modem, const char *key,
+				GAtDebugFunc debug_func, char *debug_prefix,
+				char *tty_option, ...)
+{
+	const char *device;
+	va_list args;
+	GIOChannel *channel;
+	GAtSyntax *syntax;
+	GAtChat *chat;
+	GHashTable *options = NULL;
+
+	device = ofono_modem_get_string(modem, key);
+	if (device == NULL)
+		return NULL;
+
+	if (tty_option) {
+		options = g_hash_table_new(g_str_hash, g_str_equal);
+		if (options == NULL)
+			return NULL;
+
+		va_start(args, tty_option);
+		while (tty_option) {
+			gpointer value = (gpointer) va_arg(args, const char *);
+
+			g_hash_table_insert(options, tty_option, value);
+			tty_option = (gpointer) va_arg(args, const char *);
+		}
+	}
+
+	channel = g_at_tty_open(device, options);
+
+	if (options)
+		g_hash_table_destroy(options);
+
+	if (channel == NULL)
+		return NULL;
+
+	syntax = g_at_syntax_new_gsm_permissive();
+	chat = g_at_chat_new(channel, syntax);
+	g_at_syntax_unref(syntax);
+	g_io_channel_unref(channel);
+
+	if (chat == NULL)
+		return NULL;
+
+	if (getenv("OFONO_AT_DEBUG"))
+		g_at_chat_set_debug(chat, debug_func, debug_prefix);
+
+	return chat;
+}
