@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -42,6 +41,8 @@
 #include "atmodem.h"
 #include "vendor.h"
 
+#include "network-registration.h"
+
 static const char *none_prefix[] = { NULL };
 static const char *creg_prefix[] = { "+CREG:", NULL };
 static const char *cops_prefix[] = { "+COPS:", NULL };
@@ -52,20 +53,6 @@ static const char *smoni_prefix[] = { "^SMONI:", NULL };
 static const char *zpas_prefix[] = { "+ZPAS:", NULL };
 static const char *option_tech_prefix[] = { "_OCTI:", "_OUWCTI:", NULL };
 
-struct netreg_data {
-	GAtChat *chat;
-	char mcc[OFONO_MAX_MCC_LENGTH + 1];
-	char mnc[OFONO_MAX_MNC_LENGTH + 1];
-	int signal_index; /* If strength is reported via CIND */
-	int signal_min; /* min strength reported via CIND */
-	int signal_max; /* max strength reported via CIND */
-	int signal_invalid; /* invalid strength reported via CIND */
-	int tech;
-	struct ofono_network_time time;
-	guint nitz_timeout;
-	unsigned int vendor;
-};
-
 struct tech_query {
 	int status;
 	int lac;
@@ -179,7 +166,7 @@ static int option_parse_tech(GAtResult *result)
 	return tech;
 }
 
-static int cinterion_parse_tech(GAtResult *result)
+static int gemalto_parse_tech(GAtResult *result)
 {
 	int tech = -1;
 	GAtResultIter iter;
@@ -210,7 +197,7 @@ static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_netreg_status_cb_t cb = cbd->cb;
 	int status, lac, ci, tech;
 	struct ofono_error error;
-	struct netreg_data *nd = cbd->user;
+	struct at_netreg_data *nd = cbd->user;
 
 	decode_at_error(&error, g_at_result_final_response(result));
 
@@ -228,16 +215,20 @@ static void at_creg_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if ((status == 1 || status == 5) && (tech == -1))
 		tech = nd->tech;
 
+	/* 6-10 is EUTRAN, with 8 being emergency bearer case */
+	if (status > 5 && tech == -1)
+		tech = ACCESS_TECHNOLOGY_EUTRAN;
+
 	cb(&error, status, lac, ci, tech, cbd->data);
 }
 
-static void cinterion_query_tech_cb(gboolean ok, GAtResult *result,
+static void gemalto_query_tech_cb(gboolean ok, GAtResult *result,
                                               gpointer user_data)
 {
 	struct tech_query *tq = user_data;
 	int tech;
 
-	tech = cinterion_parse_tech(result);
+	tech = gemalto_parse_tech(result);
 
 	ofono_netreg_status_notify(tq->netreg,
 			tq->status, tq->lac, tq->ci, tech);
@@ -247,7 +238,7 @@ static void zte_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	struct ofono_netreg *netreg = cbd->data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (ok)
 		nd->tech = zte_parse_tech(result);
@@ -259,7 +250,7 @@ static void option_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	struct ofono_netreg *netreg = cbd->data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (ok)
 		nd->tech = option_parse_tech(result);
@@ -267,11 +258,11 @@ static void option_tech_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		nd->tech = -1;
 }
 
-static void at_registration_status(struct ofono_netreg *netreg,
+void at_registration_status(struct ofono_netreg *netreg,
 					ofono_netreg_status_cb_t cb,
 					void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	cbd->user = nd;
@@ -334,7 +325,7 @@ static void at_registration_status(struct ofono_netreg *netreg,
 static void cops_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(cbd->user);
+	struct at_netreg_data *nd = ofono_netreg_get_data(cbd->user);
 	ofono_netreg_operator_cb_t cb = cbd->cb;
 	struct ofono_network_operator op;
 	GAtResultIter iter;
@@ -395,7 +386,7 @@ error:
 static void cops_numeric_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(cbd->user);
+	struct at_netreg_data *nd = ofono_netreg_get_data(cbd->user);
 	ofono_netreg_operator_cb_t cb = cbd->cb;
 	GAtResultIter iter;
 	const char *str;
@@ -447,10 +438,10 @@ error:
 	g_free(cbd);
 }
 
-static void at_current_operator(struct ofono_netreg *netreg,
+void at_current_operator(struct ofono_netreg *netreg,
 				ofono_netreg_operator_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	gboolean ok;
 
@@ -586,10 +577,10 @@ static void cops_list_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	g_free(list);
 }
 
-static void at_list_operators(struct ofono_netreg *netreg,
+void at_list_operators(struct ofono_netreg *netreg,
 				ofono_netreg_operator_list_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=?", cops_prefix,
@@ -612,10 +603,10 @@ static void register_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	cb(&error, cbd->data);
 }
 
-static void at_register_auto(struct ofono_netreg *netreg,
+void at_register_auto(struct ofono_netreg *netreg,
 				ofono_netreg_register_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	if (g_at_chat_send(nd->chat, "AT+COPS=0", none_prefix,
@@ -627,11 +618,11 @@ static void at_register_auto(struct ofono_netreg *netreg,
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
-static void at_register_manual(struct ofono_netreg *netreg,
+void at_register_manual(struct ofono_netreg *netreg,
 				const char *mcc, const char *mnc,
 				ofono_netreg_register_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[128];
 
@@ -720,7 +711,7 @@ static void ifx_xhomezr_notify(GAtResult *result, gpointer user_data)
 static void ifx_xreg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int state;
 	const char *band;
 	GAtResultIter iter;
@@ -819,7 +810,7 @@ static void ifx_xcsq_notify(GAtResult *result, gpointer user_data)
 static void ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int strength, ind;
 	GAtResultIter iter;
 
@@ -848,7 +839,7 @@ static void ciev_notify(GAtResult *result, gpointer user_data)
 static void telit_ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	const char *signal_identifier = "rssi";
 	const char *ind_str;
 	int strength;
@@ -876,10 +867,10 @@ static void telit_ciev_notify(GAtResult *result, gpointer user_data)
 	ofono_netreg_strength_notify(netreg, strength);
 }
 
-static void cinterion_ciev_notify(GAtResult *result, gpointer user_data)
+static void gemalto_ciev_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	const char *signal_identifier = "rssi";
 	const char *ind_str;
 	int strength;
@@ -912,7 +903,7 @@ static void cinterion_ciev_notify(GAtResult *result, gpointer user_data)
 static void ctzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	const char *tz;
 	GAtResultIter iter;
 
@@ -934,7 +925,7 @@ static void ctzv_notify(GAtResult *result, gpointer user_data)
 static void tlts_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	char tz[4];
 	const char *time;
@@ -969,7 +960,7 @@ static void tlts_notify(GAtResult *result, gpointer user_data)
 static gboolean notify_time(gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	nd->nitz_timeout = 0;
 
@@ -981,7 +972,7 @@ static gboolean notify_time(gpointer user_data)
 static void ifx_ctzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	const char *tz, *time;
 	GAtResultIter iter;
@@ -1019,7 +1010,7 @@ static void ifx_ctzv_notify(GAtResult *result, gpointer user_data)
 static void ifx_ctzdst_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int dst;
 	GAtResultIter iter;
 
@@ -1047,7 +1038,7 @@ static void cind_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	ofono_netreg_strength_cb_t cb = cbd->cb;
-	struct netreg_data *nd = cbd->user;
+	struct at_netreg_data *nd = cbd->user;
 	int index;
 	int strength;
 	GAtResultIter iter;
@@ -1101,7 +1092,7 @@ static void huawei_rssi_notify(GAtResult *result, gpointer user_data)
 static void huawei_mode_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	int mode, submode;
 
@@ -1129,7 +1120,7 @@ static void huawei_mode_notify(GAtResult *result, gpointer user_data)
 static void huawei_hcsq_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *mode;
 
@@ -1150,7 +1141,7 @@ static void huawei_hcsq_notify(GAtResult *result, gpointer user_data)
 static void huawei_nwtime_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	char tz[4];
 	const char *date, *time, *dst;
@@ -1225,10 +1216,10 @@ static void csq_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	cb(&error, strength, cbd->data);
 }
 
-static void at_signal_strength(struct ofono_netreg *netreg,
+void at_signal_strength(struct ofono_netreg *netreg,
 				ofono_netreg_strength_cb_t cb, void *data)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
 	cbd->user = nd;
@@ -1255,7 +1246,7 @@ static void at_signal_strength(struct ofono_netreg *netreg,
 static void mbm_etzv_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	int year, mon, mday, hour, min, sec;
 	const char *tz, *time, *timestamp;
 	GAtResultIter iter;
@@ -1304,7 +1295,7 @@ static void mbm_etzv_notify(GAtResult *result, gpointer user_data)
 static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	int mode, gsm, umts;
 
@@ -1356,7 +1347,7 @@ static void mbm_erinfo_notify(GAtResult *result, gpointer user_data)
 static void icera_nwstate_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *mccmnc, *tech, *state;
 	int rssi;
@@ -1424,7 +1415,7 @@ static int cnti_to_tech(const char *cnti)
 static void gobi_cnti_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *tech;
 	int option;
@@ -1449,7 +1440,7 @@ static void gobi_cnti_notify(GAtResult *result, gpointer user_data)
 static void nw_cnti_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *tech;
 	int option;
@@ -1475,7 +1466,7 @@ static void cnti_query_tech_cb(gboolean ok, GAtResult *result,
 						gpointer user_data)
 {
 	struct tech_query *tq = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(tq->netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(tq->netreg);
 
 	ofono_netreg_status_notify(tq->netreg,
 			tq->status, tq->lac, tq->ci, nd->tech);
@@ -1515,7 +1506,7 @@ static void creg_notify(GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
 	int status, lac, ci, tech;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	struct tech_query *tq;
 
 	if (at_util_parse_reg_unsolicited(result, "+CREG:", &status,
@@ -1556,12 +1547,12 @@ static void creg_notify(GAtResult *result, gpointer user_data)
 					option_query_tech_cb, tq, g_free) > 0)
 			return;
 		break;
-    case OFONO_VENDOR_CINTERION:
-              if (g_at_chat_send(nd->chat, "AT^SMONI",
-                                      smoni_prefix,
-                                      cinterion_query_tech_cb, tq, g_free) > 0)
-                      return;
-              break;
+	case OFONO_VENDOR_GEMALTO:
+		if (g_at_chat_send(nd->chat, "AT^SMONI",
+					smoni_prefix,
+					gemalto_query_tech_cb, tq, g_free) > 0)
+			return;
+		break;
 	}
 
 	g_free(tq);
@@ -1584,7 +1575,7 @@ static void at_cmer_not_supported(struct ofono_netreg *netreg)
 static void at_cmer_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (!ok) {
 		at_cmer_not_supported(netreg);
@@ -1643,7 +1634,7 @@ static inline ofono_bool_t append_cmer_element(char *buf, int *len, int cap,
 }
 
 static ofono_bool_t build_cmer_string(char *buf, int *cmer_opts,
-					struct netreg_data *nd)
+					struct at_netreg_data *nd)
 {
 	const char *ind;
 	int len = sprintf(buf, "AT+CMER=");
@@ -1652,8 +1643,11 @@ static ofono_bool_t build_cmer_string(char *buf, int *cmer_opts,
 	DBG("");
 
 	switch (nd->vendor) {
-	case OFONO_VENDOR_UBLOX_TOBY_L2:
-		/* UBX-13002752 R33: TOBY L2 doesn't support mode 2 and 3 */
+	case OFONO_VENDOR_UBLOX:
+		/* For all u-blox models, mode 3 is equivalent to mode 1;
+		 * since some models do not support setting modes 2 nor 3
+		 * (see UBX-13002752), we prefer mode 1 for all models.
+		 */
 		mode = "1";
 		break;
 	default:
@@ -1709,7 +1703,7 @@ static void at_cmer_query_cb(ofono_bool_t ok, GAtResult *result,
 				gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	int cmer_opts_cnt = 5; /* See 27.007 Section 8.10 */
 	int cmer_opts[cmer_opts_cnt];
@@ -1757,7 +1751,7 @@ error:
 static void cind_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	GAtResultIter iter;
 	const char *str;
 	char *signal_identifier = "signal";
@@ -1864,7 +1858,7 @@ error:
 static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (!ok) {
 		ofono_error("Unable to initialize Network Registration");
@@ -2028,12 +2022,12 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_send(nd->chat, "AT*TLTS=1", none_prefix,
 						NULL, NULL, NULL);
 		break;
-	case OFONO_VENDOR_CINTERION:
+	case OFONO_VENDOR_GEMALTO:
 		/*
-		 * We can't set rssi bounds from Cinterion responses
+		 * We can't set rssi bounds from Gemalto responses
 		 * so set them up to specified values here
 		 *
-		 * Cinterion rssi signal strength specified as:
+		 * Gemalto rssi signal strength specified as:
 		 * 0      <= -112dBm
 		 * 1 - 4  signal strengh in 15 dB steps
 		 * 5      >= -51 dBm
@@ -2047,7 +2041,7 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		g_at_chat_send(nd->chat, "AT^SIND=\"rssi\",1", none_prefix,
 				NULL, NULL, NULL);
 		g_at_chat_register(nd->chat, "+CIEV:",
-				cinterion_ciev_notify, FALSE, netreg, NULL);
+				gemalto_ciev_notify, FALSE, netreg, NULL);
 		break;
 	case OFONO_VENDOR_NOKIA:
 	case OFONO_VENDOR_SAMSUNG:
@@ -2067,7 +2061,7 @@ static void at_creg_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 static void at_creg_test_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_netreg *netreg = user_data;
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 	gint range[2];
 	GAtResultIter iter;
 	int creg1 = 0;
@@ -2115,9 +2109,9 @@ static int at_netreg_probe(struct ofono_netreg *netreg, unsigned int vendor,
 				void *data)
 {
 	GAtChat *chat = data;
-	struct netreg_data *nd;
+	struct at_netreg_data *nd;
 
-	nd = g_new0(struct netreg_data, 1);
+	nd = g_new0(struct at_netreg_data, 1);
 
 	nd->chat = g_at_chat_clone(chat);
 	nd->vendor = vendor;
@@ -2138,9 +2132,9 @@ static int at_netreg_probe(struct ofono_netreg *netreg, unsigned int vendor,
 	return 0;
 }
 
-static void at_netreg_remove(struct ofono_netreg *netreg)
+void at_netreg_remove(struct ofono_netreg *netreg)
 {
-	struct netreg_data *nd = ofono_netreg_get_data(netreg);
+	struct at_netreg_data *nd = ofono_netreg_get_data(netreg);
 
 	if (nd->nitz_timeout)
 		g_source_remove(nd->nitz_timeout);
@@ -2151,7 +2145,7 @@ static void at_netreg_remove(struct ofono_netreg *netreg)
 	g_free(nd);
 }
 
-static struct ofono_netreg_driver driver = {
+static const struct ofono_netreg_driver driver = {
 	.name				= "atmodem",
 	.probe				= at_netreg_probe,
 	.remove				= at_netreg_remove,
