@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -64,14 +63,10 @@ static void calypso_template(struct ofono_voicecall *vc, const char *cmd,
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
-		goto error;
-
 	if (g_at_chat_send(vd->chat, cmd, none_prefix,
 				calypso_generic_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
@@ -80,7 +75,6 @@ error:
 static void calypso_dial(struct ofono_voicecall *vc,
 				const struct ofono_phone_number *ph,
 				enum ofono_clir_option clir,
-				enum ofono_cug_option cug,
 				ofono_voicecall_cb_t cb, void *data)
 {
 	char buf[256];
@@ -101,14 +95,6 @@ static void calypso_dial(struct ofono_voicecall *vc,
 		break;
 	}
 
-	switch (cug) {
-	case OFONO_CUG_OPTION_INVOCATION:
-		strcat(buf, "G");
-		break;
-	default:
-		break;
-	}
-
 	strcat(buf, ";");
 
 	calypso_template(vc, buf, cb, data);
@@ -210,7 +196,7 @@ static void calypso_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 	/* strlen("+VTS=\"T\";") = 9 + initial AT + null */
 	buf = g_try_new(char, len * 9 + 3);
 
-	if (!buf) {
+	if (buf == NULL) {
 		CALLBACK_WITH_FAILURE(cb, data);
 		return;
 	}
@@ -327,6 +313,8 @@ static void cpi_notify(GAtResult *result, gpointer user_data)
 		g_at_chat_send(vd->chat, "AT%N0187", none_prefix,
 				NULL, NULL, NULL);
 
+	ofono_call_init(&call);
+
 	switch (msgtype) {
 	case 0:
 		/* Set call status to incoming */
@@ -390,7 +378,7 @@ static int calypso_voicecall_probe(struct ofono_voicecall *vc,
 	struct voicecall_data *vd;
 
 	vd = g_try_new0(struct voicecall_data, 1);
-	if (!vd)
+	if (vd == NULL)
 		return -ENOMEM;
 
 	vd->chat = g_at_chat_clone(chat);
@@ -413,7 +401,7 @@ static void calypso_voicecall_remove(struct ofono_voicecall *vc)
 	g_free(vd);
 }
 
-static struct ofono_voicecall_driver driver = {
+static const struct ofono_voicecall_driver driver = {
 	.name			= "calypsomodem",
 	.probe			= calypso_voicecall_probe,
 	.remove			= calypso_voicecall_remove,
@@ -434,12 +422,12 @@ static struct ofono_voicecall_driver driver = {
 	.send_tones		= calypso_send_dtmf
 };
 
-void calypso_voicecall_init()
+void calypso_voicecall_init(void)
 {
 	ofono_voicecall_driver_register(&driver);
 }
 
-void calypso_voicecall_exit()
+void calypso_voicecall_exit(void)
 {
 	ofono_voicecall_driver_unregister(&driver);
 }
