@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -31,15 +30,15 @@
 #include <unistd.h>
 
 #include <glib.h>
+#include <gatchat.h>
+#include <gatresult.h>
 
 #include <ofono/log.h>
 #include <ofono/modem.h>
 #include <ofono/call-volume.h>
 
-#include "gatchat.h"
-#include "gatresult.h"
-
 #include "hfpmodem.h"
+#include "slc.h"
 
 #define HFP_CALL_VOLUME_MAX 15
 
@@ -50,6 +49,7 @@ struct cv_data {
 	GAtChat *chat;
 	unsigned char sp_volume;
 	unsigned char mic_volume;
+	guint register_source;
 };
 
 static void cv_generic_set_cb(gboolean ok, GAtResult *result,
@@ -73,9 +73,6 @@ static void hfp_speaker_volume(struct ofono_call_volume *cv,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
 
-	if (cbd == NULL)
-		goto error;
-
 	vd->sp_volume = percent;
 
 	snprintf(buf, sizeof(buf), "AT+VGS=%d",
@@ -85,7 +82,6 @@ static void hfp_speaker_volume(struct ofono_call_volume *cv,
 				cv_generic_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
@@ -100,9 +96,6 @@ static void hfp_microphone_volume(struct ofono_call_volume *cv,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
 
-	if (cbd == NULL)
-		goto error;
-
 	vd->mic_volume = percent;
 
 	snprintf(buf, sizeof(buf), "AT+VGM=%d",
@@ -112,7 +105,6 @@ static void hfp_microphone_volume(struct ofono_call_volume *cv,
 				cv_generic_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
@@ -174,36 +166,40 @@ static void sync_microphone_volume_cb(const struct ofono_error *error,
 	ofono_call_volume_set_microphone_volume(cv, vd->mic_volume);
 }
 
-static void hfp_call_volume_initialized(gpointer user_data)
+static gboolean hfp_call_volume_register(gpointer user_data)
 {
 	struct ofono_call_volume *cv = user_data;
 	struct cv_data *vd = ofono_call_volume_get_data(cv);
 
 	DBG("");
 
+	vd->register_source = 0;
+
 	g_at_chat_register(vd->chat, "+VGS:", vgs_notify, FALSE, cv, NULL);
 	g_at_chat_register(vd->chat, "+VGM:", vgm_notify, FALSE, cv, NULL);
 
-	ofono_call_volume_register(cv);
-
 	/* set sp and mic volume at 50 percents by default */
 	hfp_speaker_volume(cv, 50, sync_speaker_volume_cb, cv);
 	hfp_microphone_volume(cv, 50, sync_microphone_volume_cb, cv);
+
+	ofono_call_volume_register(cv);
+
+	return FALSE;
 }
 
 static int hfp_call_volume_probe(struct ofono_call_volume *cv,
 					unsigned int vendor, void *data)
 {
-	struct hfp_data *d = data;
+	struct hfp_slc_info *info = data;
 	struct cv_data *vd;
 
 	DBG("");
 	vd = g_new0(struct cv_data, 1);
-	vd->chat = d->chat;
+	vd->chat = g_at_chat_clone(info->chat);
 
 	ofono_call_volume_set_data(cv, vd);
 
-	hfp_call_volume_initialized(cv);
+	vd->register_source = g_idle_add(hfp_call_volume_register, cv);
 
 	return 0;
 }
@@ -212,27 +208,30 @@ static void hfp_call_volume_remove(struct ofono_call_volume *cv)
 {
 	struct cv_data *vd = ofono_call_volume_get_data(cv);
 
+	if (vd->register_source != 0)
+		g_source_remove(vd->register_source);
+
 	ofono_call_volume_set_data(cv, NULL);
 
+	g_at_chat_unref(vd->chat);
 	g_free(vd);
 }
 
-static struct ofono_call_volume_driver driver = {
+static const struct ofono_call_volume_driver driver = {
 	.name			= "hfpmodem",
 	.probe			= hfp_call_volume_probe,
 	.remove			= hfp_call_volume_remove,
 	.speaker_volume		= hfp_speaker_volume,
-	.microphone_volume 	= hfp_microphone_volume,
+	.microphone_volume	= hfp_microphone_volume,
 	.mute			= NULL,
 };
 
-void hfp_call_volume_init()
+void hfp_call_volume_init(void)
 {
 	ofono_call_volume_driver_register(&driver);
 }
 
-void hfp_call_volume_exit()
+void hfp_call_volume_exit(void)
 {
 	ofono_call_volume_driver_unregister(&driver);
 }
-
