@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -159,8 +158,10 @@ static void opsys_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_radio_settings *rs = user_data;
 
-	if (!ok)
+	if (!ok) {
+		ofono_radio_settings_remove(rs);
 		return;
+	}
 
 	ofono_radio_settings_register(rs);
 }
@@ -197,7 +198,7 @@ static void hso_radio_settings_remove(struct ofono_radio_settings *rs)
 	g_free(rsd);
 }
 
-static struct ofono_radio_settings_driver driver = {
+static const struct ofono_radio_settings_driver driver = {
 	.name			= "hsomodem",
 	.probe			= hso_radio_settings_probe,
 	.remove			= hso_radio_settings_remove,
