@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -37,7 +37,9 @@ static int ifxmodem_init(void)
 	ifx_voicecall_init();
 	ifx_audio_settings_init();
 	ifx_radio_settings_init();
+	ifx_gprs_context_init();
 	ifx_stk_init();
+	ifx_ctm_init();
 
 	return 0;
 }
@@ -45,9 +47,11 @@ static int ifxmodem_init(void)
 static void ifxmodem_exit(void)
 {
 	ifx_stk_exit();
+	ifx_gprs_context_exit();
 	ifx_radio_settings_exit();
 	ifx_audio_settings_exit();
 	ifx_voicecall_exit();
+	ifx_ctm_exit();
 }
 
 OFONO_PLUGIN_DEFINE(ifxmodem, "Infineon modem driver", VERSION,
