@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2009-2010 Nokia Corporation and/or its subsidiary(-ies).
+ *  Copyright (C) 2009-2011  Nokia Corporation and/or its subsidiary(-ies).
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,10 +46,9 @@ struct settings_data {
 	GIsiClient *client;
 };
 
-static void update_status_mask(unsigned int *mask, int bsc)
+static void update_status_mask(uint32_t *mask, uint8_t bsc)
 {
 	switch (bsc) {
-
 	case SS_GSM_TELEPHONY:
 		*mask |= 1;
 		break;
@@ -84,104 +82,116 @@ static void update_status_mask(unsigned int *mask, int bsc)
 		break;
 
 	default:
-		DBG("Unknown BSC value %d, please report\n", bsc);
+		DBG("Unknown BSC value %d, please report", bsc);
 		break;
 	}
 }
 
-static gboolean query_resp_cb(GIsiClient *client,
-				const void *restrict data, size_t len,
-				uint16_t object, void *opaque)
+static gboolean check_resp(const GIsiMessage *msg, uint8_t msgid, uint8_t type)
 {
-	GIsiSubBlockIter iter;
-	const unsigned char *msg = data;
-	struct isi_cb_data *cbd = opaque;
-	ofono_call_settings_status_cb_t cb = cbd->cb;
-	guint32 mask = 0;
+	uint8_t service;
 
-	if (!msg) {
-		DBG("ISI client error: %d", g_isi_client_error(client));
-		goto error;
+	if (g_isi_msg_error(msg) < 0) {
+		DBG("Error: %s", g_isi_msg_strerror(msg));
+		return FALSE;
 	}
 
-	if (len < 7 || msg[0] != SS_SERVICE_COMPLETED_RESP)
+	if (g_isi_msg_id(msg) != msgid) {
+		DBG("Unexpected msg: %s",
+			ss_message_id_name(g_isi_msg_id(msg)));
+		return FALSE;
+	}
+
+	if (!g_isi_msg_data_get_byte(msg, 0, &service) || service != type) {
+		DBG("Unexpected service type: 0x%02X", service);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean decode_gsm_bsc_info(GIsiSubBlockIter *iter, uint32_t *mask)
+{
+	uint8_t *bsc;
+	uint8_t num, i;
+
+	if (!g_isi_sb_iter_get_byte(iter, &num, 2))
+		return FALSE;
+
+	if (!g_isi_sb_iter_get_struct(iter, (void **) &bsc, num, 3))
+		return FALSE;
+
+	for (i = 0; i < num; i++)
+		update_status_mask(mask, bsc[i]);
+
+	return TRUE;
+}
+
+static void status_query_cb(const GIsiMessage *msg, void *data)
+{
+	struct isi_cb_data *cbd = data;
+	ofono_call_settings_status_cb_t cb = cbd->cb;
+	GIsiSubBlockIter iter;
+
+	uint32_t mask = 0;
+	uint8_t status;
+
+	if (!check_resp(msg, SS_SERVICE_COMPLETED_RESP, SS_INTERROGATION))
 		goto error;
 
-	if (msg[1] != SS_INTERROGATION)
-		goto error;
-
-	for (g_isi_sb_iter_init(&iter, msg, len, 7);
-		g_isi_sb_iter_is_valid(&iter);
-		g_isi_sb_iter_next(&iter)) {
+	for (g_isi_sb_iter_init(&iter, msg, 6);
+			g_isi_sb_iter_is_valid(&iter);
+			g_isi_sb_iter_next(&iter)) {
 
 		switch (g_isi_sb_iter_get_id(&iter)) {
-
 		case SS_STATUS_RESULT:
+
+			if (!g_isi_sb_iter_get_byte(&iter, &status, 2))
+				goto error;
+
+			if (status & SS_GSM_PROVISIONED)
+				mask = 1;
+
 			break;
 
 		case SS_GSM_ADDITIONAL_INFO:
 			break;
 
-		case SS_GSM_BSC_INFO: {
+		case SS_GSM_BSC_INFO:
 
-			guint8 bsc;
-			guint8 count;
-			guint8 i;
-
-			if (!g_isi_sb_iter_get_byte(&iter, &count, 2))
+			if (!decode_gsm_bsc_info(&iter, &mask))
 				goto error;
 
-			for (i = 0; i < count; i++) {
-				if (!g_isi_sb_iter_get_byte(&iter, &bsc, 3 + i))
-					goto error;
-				update_status_mask(&mask, bsc);
-			}
-			break;
-		}
-		default:
-			DBG("Skipping sub-block: %s (%zd bytes)",
-				ss_subblock_name(g_isi_sb_iter_get_id(&iter)),
-				g_isi_sb_iter_get_len(&iter));
 			break;
 		}
 	}
 
-	DBG("status_mask %d\n", mask);
 	CALLBACK_WITH_SUCCESS(cb, mask, cbd->data);
-	goto out;
+	return;
 
 error:
 	CALLBACK_WITH_FAILURE(cb, 0, cbd->data);
-
-out:
-	g_free(cbd);
-	return TRUE;
-
 }
 
-static void isi_cw_query(struct ofono_call_settings *cs, int cls,
-			ofono_call_settings_status_cb_t cb, void *data)
+static void isi_clip_query(struct ofono_call_settings *cs,
+				ofono_call_settings_status_cb_t cb, void *data)
 {
 	struct settings_data *sd = ofono_call_settings_get_data(cs);
 	struct isi_cb_data *cbd = isi_cb_data_new(cs, cb, data);
-
-	unsigned char msg[] = {
+	const uint8_t msg[] = {
 		SS_SERVICE_REQ,
 		SS_INTERROGATION,
 		SS_ALL_TELE_AND_BEARER,
-		SS_GSM_CALL_WAITING >> 8,   /* Supplementary services */
-		SS_GSM_CALL_WAITING & 0xFF, /* code */
+		SS_GSM_CLIP >> 8, SS_GSM_CLIP & 0xFF,
 		SS_SEND_ADDITIONAL_INFO,
-		0 /* Subblock count */
+		0,
 	};
 
-	DBG("waiting class %d\n", cls);
-
-	if (!cbd || !sd)
+	if (sd == NULL || cbd == NULL)
 		goto error;
 
-	if (g_isi_request_make(sd->client, msg, sizeof(msg), SS_TIMEOUT,
-				query_resp_cb, cbd))
+	if (g_isi_client_send(sd->client, msg, sizeof(msg), status_query_cb,
+				cbd, g_free))
 		return;
 
 error:
@@ -189,65 +199,131 @@ error:
 	g_free(cbd);
 }
 
-static gboolean set_resp_cb(GIsiClient *client,
-				const void *restrict data, size_t len,
-				uint16_t object, void *opaque)
+static void isi_colp_query(struct ofono_call_settings *cs,
+				ofono_call_settings_status_cb_t cb, void *data)
 {
-	GIsiSubBlockIter iter;
-	const unsigned char *msg = data;
-	struct isi_cb_data *cbd = opaque;
-	ofono_call_settings_set_cb_t cb = cbd->cb;
+	struct settings_data *sd = ofono_call_settings_get_data(cs);
+	struct isi_cb_data *cbd = isi_cb_data_new(cs, cb, data);
+	const uint8_t msg[] = {
+		SS_SERVICE_REQ,
+		SS_INTERROGATION,
+		SS_ALL_TELE_AND_BEARER,
+		SS_GSM_COLP >> 8, SS_GSM_COLP & 0xFF,
+		SS_SEND_ADDITIONAL_INFO,
+		0,
+	};
 
-	if (len < 7 || msg[0] != SS_SERVICE_COMPLETED_RESP)
+	if (sd == NULL || cbd == NULL)
 		goto error;
 
-	if (msg[1] != SS_ACTIVATION && msg[1] != SS_DEACTIVATION)
-		goto error;
-
-	for (g_isi_sb_iter_init(&iter, msg, len, 7);
-		g_isi_sb_iter_is_valid(&iter);
-		g_isi_sb_iter_next(&iter)) {
-
-		switch (g_isi_sb_iter_get_id(&iter)) {
-
-		case SS_GSM_ADDITIONAL_INFO:
-			break;
-
-		case SS_GSM_DATA: {
-
-			guint8 status;
-
-			if (!g_isi_sb_iter_get_byte(&iter, &status, 2))
-				goto error;
-
-			if ((status & SS_GSM_ACTIVE)
-				&& (msg[1] == SS_DEACTIVATION))
-				goto error;
-
-			if (!(status & SS_GSM_ACTIVE)
-				&& (msg[1] == SS_ACTIVATION))
-				goto error;
-
-			break;
-		}
-		default:
-			DBG("Skipping sub-block: %s (%zd bytes)",
-				ss_subblock_name(g_isi_sb_iter_get_id(&iter)),
-				g_isi_sb_iter_get_len(&iter));
-			break;
-		}
-	}
-
-	CALLBACK_WITH_SUCCESS(cb, cbd->data);
-	goto out;
+	if (g_isi_client_send(sd->client, msg, sizeof(msg), status_query_cb,
+				cbd, g_free))
+		return;
 
 error:
-	CALLBACK_WITH_FAILURE(cb, cbd->data);
-
-out:
+	CALLBACK_WITH_FAILURE(cb, 0, data);
 	g_free(cbd);
-	return TRUE;
+}
 
+static void isi_colr_query(struct ofono_call_settings *cs,
+				ofono_call_settings_status_cb_t cb, void *data)
+{
+	struct settings_data *sd = ofono_call_settings_get_data(cs);
+	struct isi_cb_data *cbd = isi_cb_data_new(cs, cb, data);
+	const uint8_t msg[] = {
+		SS_SERVICE_REQ,
+		SS_INTERROGATION,
+		SS_ALL_TELE_AND_BEARER,
+		SS_GSM_COLR >> 8, SS_GSM_COLR & 0xFF,
+		SS_SEND_ADDITIONAL_INFO,
+		0,
+	};
+
+	if (sd == NULL || cbd == NULL)
+		goto error;
+
+	if (g_isi_client_send(sd->client, msg, sizeof(msg), status_query_cb,
+				cbd, g_free))
+		return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, 0, data);
+	g_free(cbd);
+}
+
+static void isi_cw_query(struct ofono_call_settings *cs, int cls,
+			ofono_call_settings_status_cb_t cb, void *data)
+{
+	struct settings_data *sd = ofono_call_settings_get_data(cs);
+	struct isi_cb_data *cbd = isi_cb_data_new(cs, cb, data);
+	const uint8_t msg[] = {
+		SS_SERVICE_REQ,
+		SS_INTERROGATION,
+		SS_ALL_TELE_AND_BEARER,
+		SS_GSM_CALL_WAITING >> 8, SS_GSM_CALL_WAITING & 0xFF,
+		SS_SEND_ADDITIONAL_INFO,
+		0,
+	};
+
+	if (sd == NULL || cbd == NULL)
+		goto error;
+
+	if (g_isi_client_send(sd->client, msg, sizeof(msg), status_query_cb,
+				cbd, g_free))
+		return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, 0, data);
+	g_free(cbd);
+}
+
+static gboolean check_cw_resp(const GIsiMessage *msg, uint8_t type)
+{
+	GIsiSubBlockIter iter;
+	uint8_t status;
+
+	if (!check_resp(msg, SS_SERVICE_COMPLETED_RESP, type))
+		return FALSE;
+
+	for (g_isi_sb_iter_init(&iter, msg, 6);
+			g_isi_sb_iter_is_valid(&iter);
+			g_isi_sb_iter_next(&iter)) {
+
+		if (g_isi_sb_iter_get_id(&iter) != SS_GSM_DATA)
+			continue;
+
+		if (!g_isi_sb_iter_get_byte(&iter, &status, 2))
+			return FALSE;
+
+		if (status & SS_GSM_ACTIVE)
+			return type == SS_ACTIVATION;
+		else
+			return type == SS_DEACTIVATION;
+	}
+
+	return FALSE;
+}
+
+static void cw_set_resp_cb(const GIsiMessage *msg, void *data)
+{
+	struct isi_cb_data *cbd = data;
+	ofono_call_settings_set_cb_t cb = cbd->cb;
+
+	if (check_cw_resp(msg, SS_ACTIVATION))
+		CALLBACK_WITH_SUCCESS(cb, cbd->data);
+	else
+		CALLBACK_WITH_FAILURE(cb, cbd->data);
+}
+
+static void cw_unset_resp_cb(const GIsiMessage *msg, void *data)
+{
+	struct isi_cb_data *cbd = data;
+	ofono_call_settings_set_cb_t cb = cbd->cb;
+
+	if (check_cw_resp(msg, SS_DEACTIVATION))
+		CALLBACK_WITH_SUCCESS(cb, cbd->data);
+	else
+		CALLBACK_WITH_FAILURE(cb, cbd->data);
 }
 
 static void isi_cw_set(struct ofono_call_settings *cs, int mode, int cls,
@@ -255,24 +331,21 @@ static void isi_cw_set(struct ofono_call_settings *cs, int mode, int cls,
 {
 	struct settings_data *sd = ofono_call_settings_get_data(cs);
 	struct isi_cb_data *cbd = isi_cb_data_new(cs, cb, data);
-
-	unsigned char msg[] = {
+	const uint8_t msg[] = {
 		SS_SERVICE_REQ,
 		mode ? SS_ACTIVATION : SS_DEACTIVATION,
 		SS_ALL_TELE_AND_BEARER,
-		SS_GSM_CALL_WAITING >> 8,   /* Supplementary services */
-		SS_GSM_CALL_WAITING & 0xFF, /* code */
+		SS_GSM_CALL_WAITING >> 8, SS_GSM_CALL_WAITING & 0xFF,
 		SS_SEND_ADDITIONAL_INFO,
-		0  /* Subblock count */
+		0, /* Subblock count */
 	};
 
-	DBG("waiting mode %d class %d\n", mode, cls);
-
-	if (!cbd || !sd)
+	if (cbd == NULL || sd == NULL)
 		goto error;
 
-	if (g_isi_request_make(sd->client, msg, sizeof(msg), SS_TIMEOUT,
-				set_resp_cb, cbd))
+	if (g_isi_client_send(sd->client, msg, sizeof(msg),
+				mode ? cw_set_resp_cb : cw_unset_resp_cb,
+				cbd, g_free))
 		return;
 
 error:
@@ -280,59 +353,40 @@ error:
 	g_free(cbd);
 }
 
-static gboolean isi_call_settings_register(gpointer user)
+static void reachable_cb(const GIsiMessage *msg, void *data)
 {
-	struct ofono_call_settings *cs = user;
+	struct ofono_call_settings *cs = data;
 
-	ofono_call_settings_register(cs);
-
-	return FALSE;
-}
-
-static void reachable_cb(GIsiClient *client, gboolean alive, uint16_t object,
-				void *opaque)
-{
-	struct ofono_call_settings *cs = opaque;
-	const char *debug = NULL;
-
-	if (!alive) {
-		DBG("Unable to bootstrap call settings driver");
+	if (g_isi_msg_error(msg) < 0) {
+		ofono_call_settings_remove(cs);
 		return;
 	}
 
-	DBG("%s (v%03d.%03d) reachable",
-		pn_resource_name(g_isi_client_resource(client)),
-		g_isi_version_major(client),
-		g_isi_version_minor(client));
+	ISI_RESOURCE_DBG(msg);
 
-	debug = getenv("OFONO_ISI_DEBUG");
-	if (debug && (strcmp(debug, "all") == 0 || strcmp(debug, "ss") == 0))
-		g_isi_client_set_debug(client, ss_debug, NULL);
-
-	g_idle_add(isi_call_settings_register, cs);
+	ofono_call_settings_register(cs);
 }
 
-
 static int isi_call_settings_probe(struct ofono_call_settings *cs,
 					unsigned int vendor, void *user)
 {
-	GIsiModem *idx = user;
-	struct settings_data *data;
+	GIsiModem *modem = user;
+	struct settings_data *sd;
 
-	data = g_try_new0(struct settings_data, 1);
-
-	if (!data)
+	sd = g_try_new0(struct settings_data, 1);
+	if (sd == NULL)
 		return -ENOMEM;
 
-	data->client = g_isi_client_create(idx, PN_SS);
+	sd->client = g_isi_client_create(modem, PN_SS);
 
-	if (!data->client)
+	if (sd->client == NULL) {
+		g_free(sd);
 		return -ENOMEM;
+	}
 
-	ofono_call_settings_set_data(cs, data);
+	ofono_call_settings_set_data(cs, sd);
 
-	if (!g_isi_verify(data->client, reachable_cb, cs))
-		DBG("Unable to verify reachability");
+	g_isi_client_verify(sd->client, reachable_cb, cs, NULL);
 
 	return 0;
 }
@@ -341,33 +395,34 @@ static void isi_call_settings_remove(struct ofono_call_settings *cs)
 {
 	struct settings_data *data = ofono_call_settings_get_data(cs);
 
-	if (!data)
+	ofono_call_settings_set_data(cs, NULL);
+
+	if (data == NULL)
 		return;
 
-	ofono_call_settings_set_data(cs, NULL);
 	g_isi_client_destroy(data->client);
 	g_free(data);
 }
 
-static struct ofono_call_settings_driver driver = {
+static const struct ofono_call_settings_driver driver = {
 	.name			= "isimodem",
 	.probe			= isi_call_settings_probe,
 	.remove			= isi_call_settings_remove,
-	.clip_query		= NULL,
-	.colp_query		= NULL,
+	.clip_query		= isi_clip_query,
+	.colp_query		= isi_colp_query,
+	.colr_query		= isi_colr_query,
 	.clir_query		= NULL,
-	.colr_query		= NULL,
 	.clir_set		= NULL,
 	.cw_query		= isi_cw_query,
 	.cw_set			= isi_cw_set
 };
 
-void isi_call_settings_init()
+void isi_call_settings_init(void)
 {
 	ofono_call_settings_driver_register(&driver);
 }
 
-void isi_call_settings_exit()
+void isi_call_settings_exit(void)
 {
 	ofono_call_settings_driver_unregister(&driver);
 }
