@@ -24,7 +24,6 @@
 #include <config.h>
 #endif
 
-#define _GNU_SOURCE
 #include <string.h>
 
 #include <glib.h>
@@ -270,7 +269,7 @@ error:
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
-static struct ofono_ussd_driver driver = {
+static const struct ofono_ussd_driver driver = {
 	.name		= "qmimodem",
 	.probe		= qmi_ussd_probe,
 	.remove		= qmi_ussd_remove,
