@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -57,6 +57,8 @@ static gchar *option_username = NULL;
 static gchar *option_password = NULL;
 static gchar *option_pppdump = NULL;
 static gboolean option_bluetooth = FALSE;
+static gboolean option_acfc = FALSE;
+static gboolean option_pfc = FALSE;
 
 static GAtPPP *ppp;
 static GAtChat *control;
@@ -89,6 +91,40 @@ static void power_down(gboolean ok, GAtResult *result, gpointer user_data)
 	g_main_loop_quit(event_loop);
 }
 
+static void kill_ppp(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	g_print("kill_ppp: %d\n", ok);
+
+	if (ok == FALSE)
+		return;
+
+	g_at_ppp_unref(ppp);
+	ppp = NULL;
+}
+
+static void ppp_suspend_ath0(gpointer user_data)
+{
+	g_at_chat_resume(modem);
+	g_at_chat_send(modem, "ATH0", none_prefix, kill_ppp, NULL, NULL);
+}
+
+static void resume_ppp(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	g_print("resume_ppp: %d\n", ok);
+
+	if (ok == FALSE)
+		return;
+
+	g_at_chat_suspend(modem);
+	g_at_ppp_resume(ppp);
+}
+
+static void ppp_suspend_ato0(gpointer user_data)
+{
+	g_at_chat_resume(modem);
+	g_at_chat_send(modem, "ATO0", none_prefix, resume_ppp, NULL, NULL);
+}
+
 static gboolean signal_cb(GIOChannel *channel, GIOCondition cond, gpointer data)
 {
 	static int terminated = 0;
@@ -120,6 +156,20 @@ static gboolean signal_cb(GIOChannel *channel, GIOCondition cond, gpointer data)
 
 		terminated++;
 		break;
+	case SIGUSR1:
+		if (ppp == NULL)
+			break;
+
+		g_at_ppp_set_suspend_function(ppp, ppp_suspend_ato0, NULL);
+		g_at_ppp_suspend(ppp);
+		break;
+	case SIGUSR2:
+		if (ppp == NULL)
+			break;
+
+		g_at_ppp_set_suspend_function(ppp, ppp_suspend_ath0, NULL);
+		g_at_ppp_suspend(ppp);
+		break;
 	default:
 		break;
 	}
@@ -312,24 +362,27 @@ static void connect_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	g_at_chat_suspend(modem);
 
 	/* open ppp */
-	ppp = g_at_ppp_new_from_io(io);
+	ppp = g_at_ppp_new();
 	if (ppp == NULL) {
 		g_print("Unable to create PPP object\n");
 		exit(1);
 	}
 	g_at_ppp_set_debug(ppp, gsmdial_debug, "PPP");
 
-	if (option_pppdump)
-		g_at_ppp_set_recording(ppp, option_pppdump);
-
 	g_at_ppp_set_credentials(ppp, option_username, option_password);
 
+	g_at_ppp_set_acfc_enabled(ppp, option_acfc);
+	g_at_ppp_set_pfc_enabled(ppp, option_pfc);
+
 	/* set connect and disconnect callbacks */
 	g_at_ppp_set_connect_function(ppp, ppp_connect, NULL);
 	g_at_ppp_set_disconnect_function(ppp, ppp_disconnect, NULL);
 
 	/* open the ppp connection */
-	g_at_ppp_open(ppp);
+	g_at_ppp_open(ppp, io);
+
+	if (option_pppdump)
+		g_at_ppp_set_recording(ppp, option_pppdump);
 }
 
 static void at_cgdcont_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -630,6 +683,10 @@ static GOptionEntry options[] = {
 				"Specify PPP password" },
 	{ "pppdump", 'D', 0, G_OPTION_ARG_STRING, &option_pppdump,
 				"Specify pppdump filename" },
+	{ "pfc", 0, 0, G_OPTION_ARG_NONE, &option_pfc,
+				"Use Protocol Field Compression" },
+	{ "acfc", 0, 0, G_OPTION_ARG_NONE, &option_acfc,
+				"Use Address & Control Field Compression" },
 	{ NULL },
 };
 
@@ -689,6 +746,7 @@ int main(int argc, char **argv)
 	sigemptyset(&mask);
 	sigaddset(&mask, SIGTERM);
 	sigaddset(&mask, SIGINT);
+	sigaddset(&mask, SIGUSR1);
 	sigaddset(&mask, SIGUSR2);
 	sigaddset(&mask, SIGPIPE);
 
@@ -713,7 +771,7 @@ int main(int argc, char **argv)
 	event_loop = g_main_loop_new(NULL, FALSE);
 
 	if (option_bluetooth) {
-		g_at_chat_send(control, "ATD*99", none_prefix, connect_cb,
+		g_at_chat_send(control, "ATD*99#", none_prefix, connect_cb,
 				NULL, NULL);
 	} else {
 		g_at_chat_send(control, "ATE0Q0V1", NULL, NULL, NULL, NULL);
