@@ -553,36 +553,15 @@ done:
 static GAtChat *open_device(struct ofono_modem *modem,
 				const char *key, char *debug)
 {
-	const char *device;
-	GIOChannel *channel;
-	GAtSyntax *syntax;
-	GAtChat *chat;
+	GAtChat *chat = at_util_open_device(modem, key, huawei_debug, debug,
+						NULL);
 
-	device = ofono_modem_get_string(modem, key);
-	if (device == NULL)
-		return NULL;
-
-	DBG("%s %s", key, device);
-
-	channel = g_at_tty_open(device, NULL);
-	if (channel == NULL)
-		return NULL;
-
-	syntax = g_at_syntax_new_gsm_permissive();
-	chat = g_at_chat_new(channel, syntax);
-	g_at_syntax_unref(syntax);
-
-	g_io_channel_unref(channel);
-
-	if (chat == NULL)
+	if (!chat)
 		return NULL;
 
 	g_at_chat_add_terminator(chat, "COMMAND NOT SUPPORT", -1, FALSE);
 	g_at_chat_add_terminator(chat, "TOO MANY PARAMETERS", -1, FALSE);
 
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(chat, huawei_debug, debug);
-
 	return chat;
 }
 
@@ -875,7 +854,7 @@ static void huawei_post_sim(struct ofono_modem *modem)
 
 		data->gprs = ofono_gprs_create(modem, OFONO_VENDOR_HUAWEI,
 						"atmodem", data->pcui);
-		data->gc = ofono_gprs_context_create(modem, 0,
+		data->gc = ofono_gprs_context_create(modem, OFONO_VENDOR_HUAWEI,
 						"atmodem", data->modem);
 
 		if (data->gprs && data->gc)
