@@ -29,6 +29,7 @@
 #include <glib.h>
 #include <gatchat.h>
 #include <gattty.h>
+#include <ell/ell.h>
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 #include <ofono/plugin.h>
@@ -40,10 +41,16 @@
 #include <ofono/gprs-context.h>
 #include <ofono/netmon.h>
 #include <ofono/lte.h>
+#include <ofono/sms.h>
 #include <ofono/voicecall.h>
+#include <ofono/call-forwarding.h>
+#include <ofono/call-settings.h>
+#include <ofono/call-meter.h>
+#include <ofono/call-barring.h>
+#include <ofono/message-waiting.h>
+#include <ofono/ussd.h>
 
 #include <drivers/atmodem/vendor.h>
-
 #include <drivers/ubloxmodem/ubloxmodem.h>
 
 static const char *uusbconf_prefix[] = { "+UUSBCONF:", NULL };
@@ -60,6 +67,10 @@ struct ublox_data {
 
 	const struct ublox_model *model;
 	int flags;
+
+	struct l_timeout *init_timeout;
+	int init_count;
+	guint init_cmd;
 };
 
 static void ublox_debug(const char *str, void *user_data)
@@ -96,52 +107,25 @@ static void ublox_remove(struct ofono_modem *modem)
 	g_free(data);
 }
 
-static GAtChat *open_device(struct ofono_modem *modem,
-				const char *key, char *debug)
+static void close_devices(struct ofono_modem* modem)
 {
-	const char *device;
-	GAtSyntax *syntax;
-	GIOChannel *channel;
-	GAtChat *chat;
+	struct ublox_data * data = ofono_modem_get_data(modem);
 
-	device = ofono_modem_get_string(modem, key);
-	if (device == NULL)
-		return NULL;
-
-	DBG("%s %s", key, device);
-
-	channel = g_at_tty_open(device, NULL);
-	if (channel == NULL)
-		return NULL;
-
-	syntax = g_at_syntax_new_gsm_permissive();
-	chat = g_at_chat_new(channel, syntax);
-	g_at_syntax_unref(syntax);
-
-	g_io_channel_unref(channel);
-
-	if (chat == NULL)
-		return NULL;
-
-	if (getenv("OFONO_AT_DEBUG"))
-		g_at_chat_set_debug(chat, ublox_debug, debug);
-
-	return chat;
+	g_at_chat_unref(data->aux);
+	data->aux = NULL;
+	g_at_chat_unref(data->modem);
+	data->modem = NULL;
+	ofono_modem_set_powered(modem, FALSE);
 }
 
 static void cfun_enable(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
-	struct ublox_data * data = ofono_modem_get_data(modem);
 
 	DBG("ok %d", ok);
 
 	if (!ok) {
-		g_at_chat_unref(data->aux);
-		data->aux = NULL;
-		g_at_chat_unref(data->modem);
-		data->modem = NULL;
-		ofono_modem_set_powered(modem, FALSE);
+		close_devices(modem);
 		return;
 	}
 
@@ -192,11 +176,7 @@ retry:
 		return;
 
 error:
-	g_at_chat_unref(data->aux);
-	data->aux = NULL;
-	g_at_chat_unref(data->modem);
-	data->modem = NULL;
-	ofono_modem_set_powered(modem, FALSE);
+	close_devices(modem);
 }
 
 static void query_model_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -243,39 +223,61 @@ static void query_model_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 
 fail:
-	g_at_chat_unref(data->aux);
-	data->aux = NULL;
-	g_at_chat_unref(data->modem);
-	data->modem = NULL;
-	ofono_modem_set_powered(modem, FALSE);
+	close_devices(modem);
 }
 
-static int ublox_enable(struct ofono_modem *modem)
+static void init_cmd_cb(gboolean ok, GAtResult *result, void *user_data)
 {
+	struct ofono_modem *modem = user_data;
 	struct ublox_data *data = ofono_modem_get_data(modem);
 
 	DBG("%p", modem);
 
-	data->aux = open_device(modem, "Aux", "Aux: ");
-	/* If this is a serial modem then the device may be behind
-	 * the 'Device' attribute instead...
+	if (!ok)
+		goto fail;
+
+	/* When the 'init command' succeeds, we insert an additional
+	 * delay of 1 second before proceeding with the actual
+	 * intialization of the device.  We reuse the init_timeout
+	 * instance for this, just clearing the command to indicate
+	 * that additional retries aren't necessary.
 	 */
-	if (data->aux == NULL) {
-		data->aux = open_device(modem, "Device", "Aux: ");
-		if (data->aux == NULL)
-			return -EINVAL;
+	data->init_cmd = 0;
+	data->init_count = 0;
+	l_timeout_modify_ms(data->init_timeout, 1000);
+
+	return;
+
+fail:
+	l_timeout_remove(data->init_timeout);
+	data->init_timeout = NULL;
+
+	close_devices(modem);
+}
+
+static void init_timeout_cb(struct l_timeout *timeout, void *user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct ublox_data *data = ofono_modem_get_data(modem);
+
+	DBG("%p", modem);
+
+	/* As long as init_cmd is set we need to either keep retrying
+	 * or fail everything after excessive retries
+	 */
+	if (data->init_cmd && data->init_count++ < 20) {
+		g_at_chat_retry(data->aux, data->init_cmd);
+		l_timeout_modify_ms(timeout, 1000);
+		return;
 	}
 
-	data->modem = open_device(modem, "Modem", "Modem: ");
-	if (data->modem) {
-		g_at_chat_set_slave(data->modem, data->aux);
-		g_at_chat_send(data->modem, "ATE0 +CMEE=1", none_prefix,
-						NULL, NULL, NULL);
-		g_at_chat_send(data->modem, "AT&C0", NULL, NULL, NULL, NULL);
-	}
+	l_timeout_remove(data->init_timeout);
+	data->init_timeout = NULL;
 
-	/* The modem can take a while to wake up if just powered on. */
-	g_at_chat_set_wakeup_command(data->aux, "AT\r", 1000, 11000);
+	if (data->init_cmd) {
+		ofono_error("failed to init modem after 20 attempts");
+		goto fail;
+	}
 
 	g_at_chat_send(data->aux, "ATE0", none_prefix,
 					NULL, NULL, NULL);
@@ -284,26 +286,81 @@ static int ublox_enable(struct ofono_modem *modem)
 
 	if (g_at_chat_send(data->aux, "AT+CGMM", NULL,
 				query_model_cb, modem, NULL) > 0)
-		return -EINPROGRESS;
+		return;
 
-	g_at_chat_unref(data->aux);
-	data->aux = NULL;
+fail:
+	close_devices(modem);
+}
 
-	return -EINVAL;
+static int ublox_enable(struct ofono_modem *modem)
+{
+	struct ublox_data *data = ofono_modem_get_data(modem);
+
+	DBG("%p", modem);
+
+	data->aux = at_util_open_device(modem, "Aux",
+					ublox_debug, "Aux: ", NULL);
+	/* If this is a serial modem then the device may be behind
+	 * the 'Device' attribute instead...
+	 */
+	if (data->aux == NULL) {
+		data->aux = at_util_open_device(modem, "Device",
+						ublox_debug, "Aux: ", NULL);
+		if (data->aux == NULL)
+			return -EINVAL;
+	}
+
+	data->modem = at_util_open_device(modem, "Modem",
+						ublox_debug, "Modem: ", NULL);
+	if (data->modem) {
+		g_at_chat_set_slave(data->modem, data->aux);
+		g_at_chat_send(data->modem, "ATE0 +CMEE=1", none_prefix,
+						NULL, NULL, NULL);
+		g_at_chat_send(data->modem, "AT&C0", NULL, NULL, NULL, NULL);
+	}
+
+	/*
+	 * uBlox devices present their USB interfaces well before those
+	 * interfaces are actually ready to use.  The specs say to monitor
+	 * the 'greeting text' to detect whether the device is ready to use;
+	 * unfortunately, other than for the TOBY L4, the greeting text is
+	 * not actually specified.
+	 *
+	 * What has been determined experimentally to work is to probe with
+	 * an 'AT' command until it responds and then wait an additional
+	 * second before continuing with device initialization.  Even for
+	 * the TOBY L4 where one should wait for the '+AT: READY' URC
+	 * before intialization, this seems to be sufficient; the 'READY'
+	 * indication always arrives within this time.
+	 *
+	 * (It would be more rigorous to actually wait for the 'READY'
+	 * indication, but that would require knowing the device model
+	 * before the device model is actually queried.  Do-able via
+	 * USB Product ID, but overkill when the above seems to work
+	 * reliably.)
+	 */
+
+	data->init_count = 0;
+	data->init_cmd = g_at_chat_send(data->aux, "AT", none_prefix,
+					init_cmd_cb, modem, NULL);
+	data->init_timeout = l_timeout_create_ms(500, init_timeout_cb, modem,
+							NULL);
+
+	return -EINPROGRESS;
 }
 
 static void cfun_disable(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
-	struct ublox_data *data = ofono_modem_get_data(modem);
 
 	DBG("");
 
-	g_at_chat_unref(data->aux);
-	data->aux = NULL;
+	if (!ok) {
+		ofono_error("Failed to disable modem");
+		return;
+	}
 
-	if (ok)
-		ofono_modem_set_powered(modem, FALSE);
+	close_devices(modem);
 }
 
 static int ublox_disable(struct ofono_modem *modem)
@@ -312,10 +369,10 @@ static int ublox_disable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
+	l_timeout_remove(data->init_timeout);
+
 	g_at_chat_cancel_all(data->modem);
 	g_at_chat_unregister_all(data->modem);
-	g_at_chat_unref(data->modem);
-	data->modem = NULL;
 
 	g_at_chat_cancel_all(data->aux);
 	g_at_chat_unregister_all(data->aux);
@@ -383,9 +440,9 @@ static void ublox_post_sim(struct ofono_modem *modem)
 	struct ofono_gprs *gprs;
 	struct ofono_gprs_context *gc;
 	GAtChat *chat = data->modem ? data->modem : data->aux;
+	struct ofono_message_waiting *mw;
 	const char *driver;
-	/* Toby L2: Create same number of contexts as supported PDP contexts. */
-	int ncontexts = data->flags & UBLOX_DEVICE_F_HIGH_THROUGHPUT_MODE ? 8 : 1;
+	const char *iface;
 	int variant;
 
 	DBG("%p", modem);
@@ -393,40 +450,41 @@ static void ublox_post_sim(struct ofono_modem *modem)
 	gprs = ofono_gprs_create(modem, data->vendor_family, "atmodem",
 					data->aux);
 
-	if (ublox_is_toby_l4(data->model)) {
+	iface = ofono_modem_get_string(modem, "NetworkInterface");
+	if (iface) {
 		driver = "ubloxmodem";
 		variant = ublox_model_to_id(data->model);
-	} else if (ublox_is_toby_l2(data->model)) {
-		if (data->flags & UBLOX_DEVICE_F_HIGH_THROUGHPUT_MODE) {
-			driver = "ubloxmodem";
-			variant = ublox_model_to_id(data->model);
-		} else {
-			driver = "atmodem";
-			variant = OFONO_VENDOR_UBLOX;
-		}
 	} else {
 		driver = "atmodem";
 		variant = OFONO_VENDOR_UBLOX;
 	}
 
-	while (ncontexts) {
-		gc = ofono_gprs_context_create(modem, variant, driver, chat);
-
-		if (gprs && gc)
-			ofono_gprs_add_context(gprs, gc);
-
-		--ncontexts;
-	}
+	gc = ofono_gprs_context_create(modem, variant, driver, chat);
+	if (gprs && gc)
+		ofono_gprs_add_context(gprs, gc);
 
 	ofono_lte_create(modem,
 		ublox_model_to_id(data->model), "ubloxmodem", data->aux);
+
+	ofono_sms_create(modem, 0, "atmodem", data->aux);
+
+	ofono_ussd_create(modem, 0, "atmodem", data->aux);
+	ofono_call_forwarding_create(modem, 0, "atmodem", data->aux);
+	ofono_call_settings_create(modem, 0, "atmodem", data->aux);
+	ofono_call_meter_create(modem, 0, "atmodem", data->aux);
+	ofono_call_barring_create(modem, 0, "atmodem", data->aux);
+
+	mw = ofono_message_waiting_create(modem);
+	if (mw)
+		ofono_message_waiting_register(mw);
 }
 
 static void ublox_post_online(struct ofono_modem *modem)
 {
 	struct ublox_data *data = ofono_modem_get_data(modem);
 
-	ofono_netreg_create(modem, data->vendor_family, "atmodem", data->aux);
+	ofono_netreg_create(modem,
+		ublox_model_to_id(data->model), "ubloxmodem", data->aux);
 
 	ofono_netmon_create(modem, data->vendor_family, "ubloxmodem", data->aux);
 }
