@@ -185,13 +185,16 @@ static void print_backtrace(unsigned int offset)
 		if (written < 0)
 			break;
 
-		len = read(infd[0], buf, sizeof(buf));
+		len = read(infd[0], buf, sizeof(buf) - 1);
 		if (len < 0)
 			break;
 
 		buf[len] = '\0';
 
 		pos = strchr(buf, '\n');
+		if (!pos)
+			break;
+
 		*pos++ = '\0';
 
 		if (strcmp(buf, "??") == 0) {
