@@ -170,6 +170,26 @@ static const char *sms_alphabet_to_string(enum sms_alphabet alphabet)
 		return "spanish";
 	case SMS_ALPHABET_PORTUGUESE:
 		return "portuguese";
+	case SMS_ALPHABET_BENGALI:
+		return "bengali";
+	case SMS_ALPHABET_GUJARATI:
+                return "gujarati";
+	case SMS_ALPHABET_HINDI:
+		return "hindi";
+	case SMS_ALPHABET_KANNADA:
+		return "kannada";
+	case SMS_ALPHABET_MALAYALAM:
+		return "malayalam";
+	case SMS_ALPHABET_ORIYA:
+		return "oriya";
+	case SMS_ALPHABET_PUNJABI:
+		return "punjabi";
+	case SMS_ALPHABET_TAMIL:
+		return "tamil";
+	case SMS_ALPHABET_TELUGU:
+		return "telugu";
+	case SMS_ALPHABET_URDU:
+		return "urdu";
 	case SMS_ALPHABET_DEFAULT:
 		return "default";
 	}
@@ -188,6 +208,26 @@ static gboolean sms_alphabet_from_string(const char *str,
 		*alphabet = SMS_ALPHABET_SPANISH;
 	else if (g_str_equal(str, "portuguese"))
 		*alphabet = SMS_ALPHABET_PORTUGUESE;
+	else if (g_str_equal(str, "bengali"))
+                *alphabet = SMS_ALPHABET_BENGALI;
+	else if (g_str_equal(str, "gujarati"))
+                *alphabet = SMS_ALPHABET_GUJARATI;
+	else if (g_str_equal(str, "hindi"))
+		*alphabet = SMS_ALPHABET_HINDI;
+	else if (g_str_equal(str, "kannada"))
+		*alphabet = SMS_ALPHABET_KANNADA;
+	else if (g_str_equal(str, "malayalam"))
+		*alphabet = SMS_ALPHABET_MALAYALAM;
+	else if (g_str_equal(str, "oriya"))
+		*alphabet = SMS_ALPHABET_ORIYA;
+	else if (g_str_equal(str, "punjabi"))
+		*alphabet = SMS_ALPHABET_PUNJABI;
+	else if (g_str_equal(str, "tamil"))
+		*alphabet = SMS_ALPHABET_TAMIL;
+	else if (g_str_equal(str, "telugu"))
+		*alphabet = SMS_ALPHABET_TELUGU;
+	else if (g_str_equal(str, "urdu"))
+		*alphabet = SMS_ALPHABET_URDU;
 	else
 		return FALSE;
 
@@ -782,6 +822,8 @@ static void netreg_status_watch(int status, int lac, int ci, int tech,
 	switch (status) {
 	case NETWORK_REGISTRATION_STATUS_REGISTERED:
 	case NETWORK_REGISTRATION_STATUS_ROAMING:
+	case NETWORK_REGISTRATION_STATUS_REGISTERED_SMS_EUTRAN:
+	case NETWORK_REGISTRATION_STATUS_ROAMING_SMS_EUTRAN:
 		sms->registered = TRUE;
 		break;
 	default:
